﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using HeuristicLab.Common;

namespace HeuristicLab.Services.Hive.Common.DataTransfer {
  [DataContract]
  [Serializable]
  public class Job : LightweightJob {
    [DataMember]
    public int Priority { get; set; }
    [DataMember]
    public int CoresNeeded { get; set; }
    [DataMember]
    public int MemoryNeeded { get; set; }
    [DataMember]
    public List<Guid> PluginsNeededIds { get; set; }
    [DataMember]
    public DateTime? LastHeartbeat { get; set; }
    
    public Job() {
      this.PluginsNeededIds = new List<Guid>();
    }
    protected Job(Job original, Cloner cloner) : base(original, cloner) {
      this.Priority = original.Priority;
      this.CoresNeeded = original.CoresNeeded;
      this.MemoryNeeded = original.MemoryNeeded;
      this.PluginsNeededIds = new List<Guid>(original.PluginsNeededIds);
      this.LastHeartbeat = original.LastHeartbeat;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new Job(this, cloner);
    }

    public override string ToString() {
      return string.Format("State: {0}, SlaveId: {1}, DateCreated: {2}, DateCalculated: {3}, CoresNeeded: {4}, MemoryNeeded: {5}", State, CurrentStateLog.SlaveId, DateCreated, CoresNeeded, MemoryNeeded);
    }

  }
}
